/* Sistemas Integrados TEKNISA
   Compras e Suprimentos

Nome: 8984_SUPTRGO.sql

Descrio:
Calcula o preo econmico para um determinado produto da coleta,
verificando se determinados campos foram alterados antes de efetivar a
alterao do regitro.

Criao:
12/02/2008 Marcelo Alexsandro Souza Silva
*/

IF EXISTS(SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'CALC_PRECO_ECON_PEDCOLETA'  and
            type = 'P'                 and
            u.uid = o.uid               and
            u.name = USER)
  DROP PROCEDURE CALC_PRECO_ECON_PEDCOLETA
  GO

CREATE PROCEDURE CALC_PRECO_ECON_PEDCOLETA
   @P_CDREGIAO     VARCHAR(3),
   @P_CDFORNECED   VARCHAR(21),
   @P_CDFILIAL     VARCHAR(4),
   @P_NRPEDCOLETA  VARCHAR(10)
AS

DECLARE @V_DTCOLEPREC   DATETIME;
DECLARE @V_DTPARCELA    DATETIME;
DECLARE @V_NRDIAS       NUMERIC(15,6);
DECLARE @V_NRFATOR      NUMERIC(15,6);
DECLARE @V_VRINDICE     NUMERIC(15,6);
DECLARE @V_DTULDIAMES   DATETIME;
DECLARE @V_DTULTAUX     DATETIME;
DECLARE @V_FATORPARC    NUMERIC(15,6);
DECLARE @V_VRPRECECO    NUMERIC(15,5);
DECLARE @V_VRPZOPGT     NUMERIC(15,6);
DECLARE @V_CONTA        NUMERIC(15,6);
DECLARE @V_DTPRIPARC    DATETIME;
DECLARE @V_TOTREST      NUMERIC(15,6);

DECLARE @V_VRPERDESITM   NUMERIC(6,2);
DECLARE @V_VRIMPITM      NUMERIC(16,2);
DECLARE @V_AUXVRIMPITM   NUMERIC(16,2);
DECLARE @V_VRRENDATUMAR  NUMERIC(7,3);
DECLARE @V_CDEMPRESA     VARCHAR(2);
DECLARE @V_INSCESTA      VARCHAR(16);

DECLARE @V_IDUTDESCBON   VARCHAR(1);
DECLARE @V_IDTPDESCFORN  VARCHAR(1);
DECLARE @V_VRPEDESCFORN  DOUBLE PRECISION;

DECLARE @V_CONTA_REG_INS NUMERIC(10);
DECLARE @V_TIPOOPER      VARCHAR(02);

/*VARIVEIS DO CURSOR C_TRG_ITEMPEDCOL*/
DECLARE @V_NEW_CDFILIAL        VARCHAR(4);
DECLARE @V_NEW_NRPEDCOLETA     VARCHAR(10);
DECLARE @V_NEW_CDFORNECED      VARCHAR(21);
DECLARE @V_NEW_CDREGIAO        VARCHAR(3);
DECLARE @V_NEW_CDPRODUTO       VARCHAR(10);
DECLARE @V_NEW_CDMARCA         VARCHAR(5);
DECLARE @V_NEW_VRPERDESITM     NUMERIC(6,2);
DECLARE @V_NEW_IDTPFOPGTCOL    VARCHAR(1);
DECLARE @V_NEW_CDFORMPGTO      VARCHAR(2);
DECLARE @V_NEW_VRITEMPEDCOL    NUMERIC(15,5);
DECLARE @V_NEW_NRDIAPGFXCOL    SMALLINT;
DECLARE @V_NEW_IDUTDIAPGCOL    VARCHAR(1);
DECLARE @V_NEW_NRPARCPGTCOL    SMALLINT;
DECLARE @V_NEW_VRPRECECOPEDCOL NUMERIC(15,5);
DECLARE @V_NEW_VRPERDESPEDCOL  NUMERIC(6,2);
DECLARE @V_VRPRECOITMPI        NUMERIC(15,5);
DECLARE @V_TESTENR             NUMERIC;
DECLARE @V_VRPERPIFORN         NUMERIC(15,2);
BEGIN
 	DECLARE C_TRG_ITEMPEDCOL CURSOR FOR
   SELECT
      ITEMPEDCOL.CDFILIAL,     ITEMPEDCOL.NRPEDCOLETA,     ITEMPEDCOL.CDFORNECED,
      ITEMPEDCOL.CDREGIAO,     ITEMPEDCOL.CDPRODUTO,       ITEMPEDCOL.CDMARCA,
      ITEMPEDCOL.VRITEMPEDCOL, ITEMPEDCOL.VRPRECECOPEDCOL, ITEMPEDCOL.VRPERDESPEDCOL
   FROM ITEMPEDCOL
   WHERE ITEMPEDCOL.CDFILIAL    = @P_CDFILIAL
     AND ITEMPEDCOL.NRPEDCOLETA = @P_NRPEDCOLETA
     AND ITEMPEDCOL.CDFORNECED  = @P_CDFORNECED
     AND ITEMPEDCOL.CDREGIAO    = @P_CDREGIAO
     AND ITEMPEDCOL.VRITEMPEDCOL > 0
     AND ITEMPEDCOL.IDPRECONPCOLATU = 'N';

   OPEN C_TRG_ITEMPEDCOL;
   FETCH NEXT FROM C_TRG_ITEMPEDCOL INTO @V_NEW_CDFILIAL,     @V_NEW_NRPEDCOLETA,
                                         @V_NEW_CDFORNECED,   @V_NEW_CDREGIAO,
                                         @V_NEW_CDPRODUTO,    @V_NEW_CDMARCA,
                                         @V_NEW_VRITEMPEDCOL, @V_NEW_VRPRECECOPEDCOL,
                                         @V_NEW_VRPERDESPEDCOL;

   WHILE (@@FETCH_STATUS = 0)
   BEGIN
      SET @V_TIPOOPER = '01';
      SELECT
         @V_TIPOOPER =  ISNULL(MAX(PARCMPSUP.CDTIPOOPER), '01')
      FROM
         PARCMPSUP,
         FILIAL
      WHERE
         (FILIAL.CDFILIAL    = @V_NEW_CDFILIAL) AND
         (PARCMPSUP.CDFILIAL = ISNULL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL));

      SET @V_DTCOLEPREC = NULL;
      SELECT @V_DTCOLEPREC = MAX(DTPEDCOLETA) -- Busca a data da Coleta
      FROM PEDCOLETA
      WHERE PEDCOLETA.CDREGIAO    = @V_NEW_CDFILIAL
        AND PEDCOLETA.CDFORNECED  = @V_NEW_CDFORNECED
        AND PEDCOLETA.CDFILIAL    = @V_NEW_CDFILIAL
        AND PEDCOLETA.NRPEDCOLETA = @V_NEW_NRPEDCOLETA;

      SET @V_CDEMPRESA = NULL;
      SELECT
        @V_CDEMPRESA = MAX(E.CDEMPRESA)
      FROM
        EMITENTEAF E, PRODUTO P, PRODUTO P1
      WHERE
        P1.CDPRODUTO = @V_NEW_CDPRODUTO AND
        P.CDPRODUTO  = E.CDPRODUTO AND
        E.CDFILIAL   = @V_NEW_CDFILIAL AND
        P1.CDARVPROD LIKE P.CDARVPROD + '%';

      IF @V_CDEMPRESA IS NULL
      BEGIN
         SELECT
            @V_CDEMPRESA = MAX(PARCMPSUP.CDEMPRESA)
         FROM
            PARCMPSUP
         WHERE
            PARCMPSUP.CDFILIAL = @V_NEW_CDFILIAL;
      END;

      SET @V_INSCESTA = NULL;
      SELECT
        @V_INSCESTA = MAX(FILIAL.CDINSCESTA)
      FROM
        FILIAL
      WHERE
        FILIAL.CDFILIAL  = @V_NEW_CDFILIAL AND
        FILIAL.CDEMPRESA = @V_CDEMPRESA;

      EXEC IMPOSTO_ITEMPEDCOL @V_CDEMPRESA,       @V_INSCESTA,       @V_NEW_CDFILIAL,
                              @V_NEW_NRPEDCOLETA, @V_NEW_CDFORNECED, @V_NEW_CDREGIAO,
                              @V_NEW_CDPRODUTO,   @V_NEW_CDMARCA,    @V_DTCOLEPREC,
                              @V_NEW_VRITEMPEDCOL,@V_NEW_VRPERDESPEDCOL;

      -- Busca o total do imposto calculado para o item da coleta
      SET @V_VRIMPITM = NULL;
      SELECT
        @V_VRIMPITM = SUM(VRALIQIMPPEDCOL)
      FROM
        IMPPEDCOLETA
      WHERE
        CDFILIAL    = @V_NEW_CDFILIAL    AND
        NRPEDCOLETA = @V_NEW_NRPEDCOLETA AND
        CDFORNECED  = @V_NEW_CDFORNECED  AND
        CDREGIAO    = @V_NEW_CDREGIAO    AND
        CDPRODUTO   = @V_NEW_CDPRODUTO   AND
        CDMARCA     = @V_NEW_CDMARCA;

      IF @V_VRIMPITM IS NULL
      BEGIN
        SET @V_VRIMPITM = 0;
      END;

    /* Clculo para os impostos com as seguintes condies(ICMS por exemplo):
      Destacar Valor, Imposto Recupervel e com Crdito de Imposto */
    /* INICIO */

      SET @V_AUXVRIMPITM = NULL;
      SELECT
        @V_AUXVRIMPITM = SUM(VRIMPPEDCOL)
      FROM
        IMPPEDCOLETA,
        IMPOSTO,
        INCIDENCIA LEFT JOIN PRODUTO
                   ON INCIDENCIA.CDPRODUTO = PRODUTO.CDPRODUTO,
        FORNECEDOR
      WHERE
        IMPPEDCOLETA.CDFILIAL    = @V_NEW_CDFILIAL         AND
        IMPPEDCOLETA.NRPEDCOLETA = @V_NEW_NRPEDCOLETA      AND
        IMPPEDCOLETA.CDFORNECED  = @V_NEW_CDFORNECED       AND
        IMPPEDCOLETA.CDREGIAO    = @V_NEW_CDREGIAO         AND
        IMPPEDCOLETA.CDPRODUTO   = @V_NEW_CDPRODUTO        AND
        IMPPEDCOLETA.CDMARCA     = @V_NEW_CDMARCA          AND
        IMPPEDCOLETA.CDIMPOSTO   = IMPOSTO.CDIMPOSTO       AND
        IMPOSTO.IDTRATIMPO       = 'V'                     AND
        IMPOSTO.IDIMPORECUP      = 'S'                     AND
        IMPOSTO.CDIMPOSTO        = INCIDENCIA.CDIMPOSTO    AND
        IMPPEDCOLETA.CDFORNECED  = FORNECEDOR.CDFORNECED   AND
        ISNULL(INCIDENCIA.CDGRUPOFISC,'') = ISNULL(FORNECEDOR.CDGRUPOFISC,'') AND
        INCIDENCIA.IDENTRSAIDIN = 'E'                  AND
        INCIDENCIA.CDEMPRESA    = @V_CDEMPRESA         AND
        INCIDENCIA.CDTIPOOPER   = @V_TIPOOPER          AND
        INCIDENCIA.CDINSCESTA   = @V_INSCESTA          AND
        INCIDENCIA.CDFISCAL     = '1'                  AND
        ISNULL(PRODUTO.CDARVPROD, '_') =
         (SELECT
            MAX(ISNULL(P.CDARVPROD, '_'))
          FROM
            INCIDENCIA I LEFT JOIN PRODUTO P
                         ON P.CDPRODUTO = I.CDPRODUTO,
            PRODUTO PR
          WHERE
            PR.CDPRODUTO   = IMPPEDCOLETA.CDPRODUTO AND
            I.CDIMPOSTO    = INCIDENCIA.CDIMPOSTO   AND
            I.IDENTRSAIDIN = 'E'                    AND
            I.CDEMPRESA    = @V_CDEMPRESA           AND
            I.CDTIPOOPER   = @V_TIPOOPER            AND
            I.CDINSCESTA   = @V_INSCESTA            AND
            I.CDFISCAL     = '1'                    AND
            ISNULL(I.CDGRUPOFISC,'') = ISNULL(FORNECEDOR.CDGRUPOFISC,'') AND            
            CHARINDEX(ISNULL(RTRIM(P.CDARVPROD),PR.CDARVPROD), PR.CDARVPROD) = 1);

      IF @V_AUXVRIMPITM IS NULL
      BEGIN
         SET @V_AUXVRIMPITM = 0;
      END;

      SET @V_VRIMPITM = @V_VRIMPITM - @V_AUXVRIMPITM;

      /* FIM */

      -- Busca o valor de redimento para o produto e marca da coleta
      SET @V_VRRENDATUMAR = NULL;
      SELECT
        @V_VRRENDATUMAR = MAX(QFOMARPROD.VRRENDATU)
      FROM QFOMARPROD
      WHERE
        QFOMARPROD.CDPRODUTO  = @V_NEW_CDPRODUTO  AND
        QFOMARPROD.CDMARCA    = @V_NEW_CDMARCA;

      IF (@V_NEW_VRPERDESITM IS NULL)
      BEGIN
         SET @V_VRPERDESITM = 0;
      END
      ELSE
      BEGIN
         SET @V_VRPERDESITM = @V_NEW_VRPERDESITM;
      END;

      SELECT
         @V_IDUTDESCBON = ISNULL(MAX(IDUTDESCBON), 'N')
      FROM
         PARASUPRI
      WHERE
         CDFILIAL = @V_NEW_CDFILIAL;

      SELECT
         @V_IDTPDESCFORN     = ISNULL(MAX(IDTPDESCFORN), 'C'),
         @V_VRPEDESCFORN     = MAX(VRPEDESCFORN),
         @V_NEW_IDTPFOPGTCOL = MAX(IDTPFORPGTFO),
         @V_NEW_CDFORMPGTO   = MAX(NRPARCPGTOFO),
         @V_NEW_NRDIAPGFXCOL = MAX(NRDIAPGFIXFO),
         @V_NEW_IDUTDIAPGCOL = MAX(IDUTDIAPGTFO),
         @V_NEW_NRPARCPGTCOL = MAX(NRPARCPGTOFO)
      FROM
         FORNECEDOR
      WHERE
         CDFORNECED = @V_NEW_CDFORNECED;

      IF @V_IDUTDESCBON = 'S' BEGIN
         IF @V_IDTPDESCFORN = 'B' BEGIN
            IF (@V_VRPEDESCFORN IS NOT NULL) AND (@V_VRPEDESCFORN <> 0) BEGIN
               SET @V_VRPERDESITM = @V_VRPEDESCFORN;
            END;
         END;
      END;

      IF (@V_VRRENDATUMAR IS NULL) OR (@V_VRRENDATUMAR = 0)
      BEGIN
         SET @V_VRRENDATUMAR = 1;
      END;
      SET @V_VRPRECECO = 0;

      --*** Se forma de pagamento for Variavel
      IF @V_NEW_IDTPFOPGTCOL = 'V'
      BEGIN
        -- Busca a data da primeira parcela se basendo na media de dias entre uma
        -- parcela e outra.
        SET @V_DTPRIPARC = NULL;
        SELECT
         @V_DTPRIPARC = MIN(ROUND((CONVERT(NUMERIC,ISNULL(DIAPGTO.NRDIAFINPGTO,1)) - 1)/2,0) +
         @V_DTCOLEPREC)
        FROM
         DIAPGTO
        WHERE
         DIAPGTO.CDFORMPGTO               = @V_NEW_CDFORMPGTO AND
         CONVERT(NUMERIC, DIAPGTO.NRDIAINIPGTO ) = 1;

        IF @V_DTPRIPARC IS NULL
        BEGIN
           SET @V_DTPRIPARC = @V_DTCOLEPREC;
        END;

        SET @V_TOTREST = 0;
        -- Cursor com as datas e percentuais de pagamento para forma de pgto variavel
        DECLARE C_PARCELAS CURSOR FOR
        SELECT
         @V_DTPRIPARC +
         PRAZOPGTFORN.NRDIASPGTOFO AS DTPARCELA,
         PRAZOPGTFORN.VRPERCPGTOFO AS VRPZOPGT
        FROM
         PRAZOPGTFORN
        WHERE
         PRAZOPGTFORN.CDFORNECED = @V_NEW_CDFORNECED;

        OPEN C_PARCELAS;
        FETCH NEXT FROM C_PARCELAS INTO @V_DTPARCELA, @V_VRPZOPGT;
        WHILE (@@FETCH_STATUS = 0)
        BEGIN

           SET @V_NRFATOR    = 1;
           SET @V_DTULDIAMES = (CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,@V_DTCOLEPREC))) + '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, @V_DTCOLEPREC)))) - 1);
           SET @V_NRDIAS     = DATEDIFF(DD, @V_DTCOLEPREC, (CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,@V_DTCOLEPREC))) + '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, @V_DTCOLEPREC)))) - 1));
           SET @V_DTULTAUX   = @V_DTCOLEPREC;

           WHILE @V_DTPARCELA > @V_DTULDIAMES
           BEGIN
             -- Busca o indice a ser aplicado para correcao
             SET @V_VRINDICE = NULL;
             SELECT @V_VRINDICE = MAX(MOVINDICE.VRINDICE)
             FROM
              PARASUPRI, MOVINDICE
             WHERE
              PARASUPRI.CDFILIAL   = @V_NEW_CDFILIAL           AND
              MONTH(MOVINDICE.DTREFIND) = MONTH(@V_DTULDIAMES) AND
              YEAR(MOVINDICE.DTREFIND)  = YEAR(@V_DTULDIAMES)  AND
              PARASUPRI.CDTPINDICE = MOVINDICE.CDTPINDICE AND
              PARASUPRI.IDPERVALOR = MOVINDICE.IDPERVALOR AND
              PARASUPRI.CDINDICE   = MOVINDICE.CDINDICE;
             IF @V_VRINDICE IS NULL
             BEGIN
                SET @V_VRINDICE = 0;
             END;
             SET @V_NRFATOR = @V_NRFATOR * POWER((1 + (@V_VRINDICE/100)), (@V_NRDIAS/30));
             SET @V_DTULTAUX = @V_DTULDIAMES;
             SET @V_DTULDIAMES = (CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,(@V_DTULDIAMES+1)))) + '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, (@V_DTULDIAMES+1))))) - 1);
             SET @V_NRDIAS = DATEDIFF(DD, @V_DTULTAUX, @V_DTULDIAMES);
           END;

           SET @V_NRDIAS = DATEDIFF(DD, @V_DTULTAUX, @V_DTPARCELA);
           -- Busca o indice a ser aplicado para correcao
           SET @V_VRINDICE = NULL;
           SELECT @V_VRINDICE = MAX(MOVINDICE.VRINDICE)
           FROM
            PARASUPRI, MOVINDICE
           WHERE
            PARASUPRI.CDFILIAL        = @V_NEW_CDFILIAL      AND
            MONTH(MOVINDICE.DTREFIND) = MONTH(@V_DTPARCELA)  AND
            YEAR(MOVINDICE.DTREFIND)  = YEAR(@V_DTPARCELA)   AND
            PARASUPRI.CDTPINDICE      = MOVINDICE.CDTPINDICE AND
            PARASUPRI.IDPERVALOR      = MOVINDICE.IDPERVALOR AND
            PARASUPRI.CDINDICE        = MOVINDICE.CDINDICE;
           IF @V_VRINDICE IS NULL
           BEGIN
              SET @V_VRINDICE = 0;
           END;
           SET @V_NRFATOR = @V_NRFATOR * POWER((1 + (@V_VRINDICE/100)),(@V_NRDIAS/30));
           SET @V_TOTREST = @V_TOTREST + ((1 / @V_NRFATOR) * (@V_VRPZOPGT/100));
           FETCH NEXT FROM C_PARCELAS INTO @V_DTPARCELA, @V_VRPZOPGT;
        END;
        CLOSE C_PARCELAS;
        DEALLOCATE C_PARCELAS;
        SET @V_VRPRECECO = ROUND((@V_NEW_VRITEMPEDCOL - (@V_NEW_VRITEMPEDCOL * (@V_VRPERDESITM /100)) + @V_VRIMPITM) *
                          (1/@V_VRRENDATUMAR) * @V_TOTREST,5);

      --*** Se forma de pagamento for Fixa
      END
      ELSE IF @V_NEW_IDTPFOPGTCOL = 'F'
      BEGIN
         SET @V_TESTENR = @V_NEW_NRDIAPGFXCOL;
         IF @V_NEW_NRDIAPGFXCOL > DAY(DATEADD(MM, 1, @V_DTCOLEPREC) - DAY(@V_DTCOLEPREC))
         BEGIN
           SET @V_TESTENR = DAY(DATEADD(MM, 1, @V_DTCOLEPREC) - DAY(@V_DTCOLEPREC));
         END;

         -- Apura a data inicial de pagamento baseado na condicao informada
         IF (CONVERT(DATETIME,RIGHT(REPLICATE('0',2)+CONVERT(VARCHAR, @V_NEW_NRDIAPGFXCOL),2)  + '/' +
            RIGHT(REPLICATE('0',2)+MONTH(@V_DTCOLEPREC),2) + '/' + RIGHT(REPLICATE('0',2)+YEAR(@V_DTCOLEPREC),2)) >
            @V_DTCOLEPREC) AND (@V_NEW_IDUTDIAPGCOL = 'I')
         BEGIN

            SET @V_DTPARCELA = CONVERT(DATETIME, RIGHT(REPLICATE('0',2) + CONVERT(VARCHAR, @V_NEW_NRDIAPGFXCOL),2) + '/' +
                                                 RIGHT(REPLICATE('0',2) + MONTH(@V_DTCOLEPREC),2) + '/' +
                                                 RIGHT(REPLICATE('0',2) + YEAR(@V_DTCOLEPREC),2));
         END;
         IF ((CONVERT(DATETIME, RIGHT(REPLICATE('0',2) + CONVERT(VARCHAR, @V_NEW_NRDIAPGFXCOL),2) + '/' +
                               RIGHT(REPLICATE('0',2) + MONTH(@V_DTCOLEPREC),2) + '/' +
                               RIGHT(REPLICATE('0',2) + YEAR(@V_DTCOLEPREC),2)) <=
             @V_DTCOLEPREC) AND (@V_NEW_IDUTDIAPGCOL = 'I')) OR
            (@V_NEW_IDUTDIAPGCOL = 'E')
         BEGIN

            SET @V_DTPARCELA = @V_NEW_NRDIAPGFXCOL + CONVERT(DATETIME, '01/' +
                               CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,@V_DTCOLEPREC))) +
                               '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, @V_DTCOLEPREC)))) - 1;

         END;

         SET @V_FATORPARC = 0;
         SET @V_CONTA     = 1;
         SET @V_NRFATOR   = 1;
         SET @V_DTULTAUX  = @V_DTCOLEPREC;

         -- Calcula fator de correcao de acordo com o numero de parcelas informadas.
         SET @V_CONTA = 1;
         WHILE @V_CONTA <= @V_NEW_NRPARCPGTCOL
         BEGIN
            SET @V_DTULDIAMES = CONVERT(DATETIME, '01/' +
                                CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,@V_DTCOLEPREC))) +
                                '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, @V_DTCOLEPREC)))) - 1;
            SET @V_NRDIAS = DATEDIFF(DD, @V_DTCOLEPREC, (CONVERT(DATETIME, '01/' +
                            CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,@V_DTCOLEPREC))) +
                            '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, @V_DTCOLEPREC)))) - 1));
            SET @V_DTULTAUX = @V_DTCOLEPREC;

            IF @V_DTULDIAMES = @V_DTCOLEPREC BEGIN
               WHILE @V_DTPARCELA > @V_DTULDIAMES
               BEGIN
                 -- Busca o indice a ser aplicado para correcao
                 SET @V_VRINDICE = NULL;
                 SELECT @V_VRINDICE = MAX(MOVINDICE.VRINDICE)
                 FROM
                  PARASUPRI, MOVINDICE
                 WHERE
                  PARASUPRI.CDFILIAL   = @V_NEW_CDFILIAL           AND
                  MONTH(MOVINDICE.DTREFIND) = MONTH(@V_DTULDIAMES) AND
                  YEAR(MOVINDICE.DTREFIND)  = YEAR(@V_DTULDIAMES)  AND
                  PARASUPRI.CDTPINDICE = MOVINDICE.CDTPINDICE AND
                  PARASUPRI.IDPERVALOR = MOVINDICE.IDPERVALOR AND
                  PARASUPRI.CDINDICE   = MOVINDICE.CDINDICE;

                 IF @V_VRINDICE IS NULL
                 BEGIN
                    SET @V_VRINDICE = 0;
                 END;
                 SET @V_NRFATOR = @V_NRFATOR * POWER((1 + (@V_VRINDICE/100)),(@V_NRDIAS/30));
                 SET @V_DTULTAUX = @V_DTULDIAMES;
                 SET @V_DTULDIAMES = CONVERT(DATETIME, '01/' +
                                     CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,(@V_DTULDIAMES+1)))) +
                                     '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, (@V_DTULDIAMES+1))))) - 1;
                 SET @V_NRDIAS = DATEDIFF(DD, @V_DTULTAUX, @V_DTULDIAMES);

               END;
            END
            ELSE BEGIN
               WHILE @V_DTPARCELA > @V_DTULDIAMES
               BEGIN
                 -- Busca o indice a ser aplicado para correcao
                 SET @V_VRINDICE = NULL;
                 SELECT @V_VRINDICE = MAX(MOVINDICE.VRINDICE)
                 FROM
                  PARASUPRI, MOVINDICE
                 WHERE
                  PARASUPRI.CDFILIAL   = @V_NEW_CDFILIAL           AND
                  MONTH(MOVINDICE.DTREFIND) = MONTH(@V_DTULDIAMES) AND
                  YEAR(MOVINDICE.DTREFIND)  = YEAR(@V_DTULDIAMES)  AND
                  PARASUPRI.CDTPINDICE = MOVINDICE.CDTPINDICE AND
                  PARASUPRI.IDPERVALOR = MOVINDICE.IDPERVALOR AND
                  PARASUPRI.CDINDICE   = MOVINDICE.CDINDICE;

                 IF @V_VRINDICE IS NULL
                 BEGIN
                    SET @V_VRINDICE = 0;
                 END;
                 SET @V_NRFATOR = @V_NRFATOR * POWER((1 + (@V_VRINDICE/100)),(@V_NRDIAS/30));
                 SET @V_DTULTAUX = @V_DTULDIAMES;
                 SET @V_DTULDIAMES = CONVERT(DATETIME, '01/' +
                                     CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,(@V_DTULDIAMES+1)))) +
                                     '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, (@V_DTULDIAMES+1))))) - 1;
                 SET @V_NRDIAS = DATEDIFF(DD, @V_DTULDIAMES, (CONVERT(DATETIME, '01/' + CONVERT(VARCHAR, DATEPART(MM, DATEADD(MM,1,(@V_DTULDIAMES+1)))) +
                                 '/' + CONVERT(VARCHAR, DATEPART(YYYY, DATEADD(MM,1, (@V_DTULDIAMES+1))))) - 1));

               END;
            END;
            SET @V_NRDIAS = DATEDIFF(DD, @V_DTULTAUX, @V_DTPARCELA);
            -- Busca o indice a ser aplicado para correcao
            SET @V_VRINDICE = NULL;
            SELECT @V_VRINDICE = MAX(MOVINDICE.VRINDICE)
            FROM
             PARASUPRI, MOVINDICE
            WHERE
             PARASUPRI.CDFILIAL        = @V_NEW_CDFILIAL      AND
             MONTH(MOVINDICE.DTREFIND) = MONTH(@V_DTPARCELA)  AND
             YEAR(MOVINDICE.DTREFIND)  = YEAR(@V_DTPARCELA)   AND
             PARASUPRI.CDTPINDICE      = MOVINDICE.CDTPINDICE AND
             PARASUPRI.IDPERVALOR      = MOVINDICE.IDPERVALOR AND
             PARASUPRI.CDINDICE        = MOVINDICE.CDINDICE;
            IF @V_VRINDICE IS NULL
            BEGIN
               SET @V_VRINDICE = 0;
            END;
            SET @V_NRFATOR = @V_NRFATOR * POWER((1 + (@V_VRINDICE/100)),(@V_NRDIAS/30));
            SET @V_FATORPARC = @V_FATORPARC + (1/@V_NRFATOR);
            SET @V_DTPARCELA = DATEADD(MM,1,@V_DTPARCELA);
            SET @V_CONTA = @V_CONTA + 1;
         END;
         SET @V_VRPRECECO = ROUND(((@V_NEW_VRITEMPEDCOL -
                            (@V_NEW_VRITEMPEDCOL * (@V_VRPERDESITM /100)) +
                            @V_VRIMPITM) / @V_NEW_NRPARCPGTCOL) *
                            (1/@V_VRRENDATUMAR) *
                            @V_FATORPARC,5);

      END
      ELSE IF @V_NEW_IDTPFOPGTCOL IS NULL
      BEGIN
      -- Se nao foi informado forma de pagamento
          SET @V_VRPRECECO = ROUND((@V_NEW_VRITEMPEDCOL -
                            (@V_NEW_VRITEMPEDCOL * (@V_VRPERDESITM /100)) +
                            @V_VRIMPITM) *
                            (1/@V_VRRENDATUMAR),5);
      END;
      SET @V_VRPERPIFORN = 0;
      SELECT @V_VRPERPIFORN = ISNULL(MAX(VRPERPIPRFO), 0)   /* PROCURA O PI NA PRODFORNCOMP*/
      FROM PRODFORNCOMP
      WHERE CDFORNECED  = @V_NEW_CDFORNECED AND
            CDPRODCOMPL = @V_NEW_CDPRODUTO;
      IF @V_VRPERPIFORN = 0 BEGIN
         SET @V_VRPERPIFORN = 0;
         SELECT @V_VRPERPIFORN = ISNULL(MAX(VRPERPIFORN), 0)  /* SE NAO ACHAR NA PRODFORNCOMP ENTAO*/
         FROM FORNECEDOR                                    /*PROCURA NO FORNECEDOR*/
         WHERE CDFORNECED = @V_NEW_CDFORNECED;
      END;
      SET @V_VRPRECOITMPI = @V_VRPRECECO - (@V_NEW_VRITEMPEDCOL*@V_VRPERPIFORN)/100;/*CALCULO DO PI(PURCHASE INCOME)*/

      UPDATE ITEMPEDCOL
      SET VRPRECECOPEDCOL = @V_VRPRECECO,
          VRPRECPIPEDCOL  = @V_VRPRECOITMPI,
          IDPRECONPCOLATU = 'S'
      WHERE CDFILIAL     = @V_NEW_CDFILIAL    AND
            NRPEDCOLETA  = @V_NEW_NRPEDCOLETA AND
            CDFORNECED   = @V_NEW_CDFORNECED  AND
            CDREGIAO     = @V_NEW_CDREGIAO    AND
            CDPRODUTO    = @V_NEW_CDPRODUTO   AND
            CDMARCA      = @V_NEW_CDMARCA;

      FETCH NEXT FROM C_TRG_ITEMPEDCOL INTO @V_NEW_CDFILIAL,     @V_NEW_NRPEDCOLETA,
                                            @V_NEW_CDFORNECED,   @V_NEW_CDREGIAO,
                                            @V_NEW_CDPRODUTO,    @V_NEW_CDMARCA,
                                            @V_NEW_VRITEMPEDCOL, @V_NEW_VRPRECECOPEDCOL,
                                            @V_NEW_VRPERDESPEDCOL;
   END;
   CLOSE C_TRG_ITEMPEDCOL;
   DEALLOCATE C_TRG_ITEMPEDCOL;
END;
GO


-- VERSAO --
EXEC ATUVEROBJ 'CALC_PRECO_ECON_PEDCOLETA','5.05.005';
GO

