// FILE: ./assets/js/utilities/PAR_Utilities.js
function PAR_Utilities(ZhPreferences, ZhFilterPreferences, templateManager, ScreenService) {

    var self = this;
    /**
     * Define o filtro(regra) nos fields definidos de acordo
     * com a revenda do operador logado.
     * 
     * @param {object} objects array com os objetos a receberem
     * a regra.
     */
    this.setResaleRulesInRestrictObjectsByLoggedOperatorResale = function(objects, loggedOperatorInfos) {
        if (this.isResaleLoggedOperator(loggedOperatorInfos)) {
            objects.forEach(function(field) {
                this.setObjectFilterWhenResaleLoggedOperator(field, loggedOperatorInfos.NRREVENDA);    
            }.bind(this));
        }
    };


    this.startFilter = function(widget, currentRow){
        ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result) {
            if (result) {
                ZhFilterPreferences.openFilterWidget(widget, currentRow);
            }
        });
    };

    this.applyDefaultFilterOnEnter = function(widget) {
        var time = 500;
        return setTimeout(function() {
            self.setDefaultFilterOnEnter(widget);
        }, time);
    };

    this.initializeFieldSelectAutocomplete = function (widgetName, field) {
        var widget = templateManager.container.getWidget(widgetName);
        widget.getField(field).initializeField();
    };
    
    this.setDefaultFilterOnEnter = function(widget){
        if (widget.inListMode()) {
            if (self.isAppliedFilter(widget)) {
                widget.reload();
            } else if (widget.currentRowDefaultFilter) {
                self.startFilter(widget, widget.currentRowDefaultFilter);
            } else {
                self.startFilter(widget, widget.currentRow);
            }
        }
    };

    this.isAppliedFilter = function(widget){
        return !!widget.aclAppliedFilter;
    };
    /**
     * Defini os filtros de revenda do obejto recebido
     * como parâmetro (field || widget).
     * 
     * @param {object} objeto object a ser modificado.
     */
    this.setObjectFilterWhenResaleLoggedOperator = function(object) {
        this.changeObjectOperTypeFilterToResaleOper(object);
        this.setObjectResaleFilter(object);
    };
    
    /**
     * Retorna um valor lógico apontando se o operador
     * logado pertence a uma revenda.
     * 
     */
    this.isResaleLoggedOperator = function(loggedOperatorInfos) {
        return (Boolean(loggedOperatorInfos.NRREVENDA) && loggedOperatorInfos.IDTPOPERADOR === 'R');
    };
    
    /**
     * Altera o filtro de tipo de operador do objeto
     * recebido como parâmetro.
     * 
     * @param {object} object objeto a ser modificado.
     */
    this.changeObjectOperTypeFilterToResaleOper = function(object) {
        var objectFilter = object.dataSourceFilter;
        var idtooperadorFilterIndex = this.getObjectPositionInArrayByName(objectFilter, 'IDTPOPERADOR');
        var resaleTypeOperator = 'R';
        
        if (idtooperadorFilterIndex === 0 || Boolean(idtooperadorFilterIndex)){
            objectFilter[idtooperadorFilterIndex].value = resaleTypeOperator;    
        }
    };
    
    /**
     * Procedimento para recuperar a posição
     * de um objeto dentro de um array.
     * 
     * @param {object} array array onde será feito
     * a varredura.
     * 
     * @param {object} objectName nome do objeto a ser
     * buscado.
     */
    this.getObjectPositionInArrayByName = function(array, objectName){
        var indexFieldObject;
        
        array.forEach(function(value, i) {
            if (value.name === objectName) {
                indexFieldObject = i;
                return;
            } 
        });
        
        return indexFieldObject;
    };
    
    /**
     * Define um filtro no objeto recebido como parâmetro
     * atribuindo o número da revenda o qual o operador
     * logado pertence.
     * 
     * @param {object} object objeto a ser modificado.
     */
    this.setObjectResaleFilter = function(object, nrrevenda) {
        var resaleFilter = {
            name: "NRREVENDA",
            operator: "=",
            value: nrrevenda
        };
        
        object.dataSourceFilter.push(resaleFilter);
    };
    
    /**
     * Seta um elemento em uma posição definida dentro de um
     * array.
     * 
     * @param {all} element elemento a ser inserido no array.
     * 
     * @param {object} array array a ser modificado.
     * 
     * @param {int} position posição que o elemento entrará no
     * array.
     * 
     * @return {object} array modificado.
     */
    this.setElementOnArrayDefinedPosition = function(element, array, position){
        if (!Boolean(position)) {
            array.push(element);
        }
        else {
            array.splice(position - 1, 0, element);    
        }
        
        return array;
    };
    
    /**
     * Função para remover um objeto do array(filtro)
     * recebido como parâmetro.
     * 
     * @param {object} filter array com os objetos(filtros).
     * 
     * @param {int} objectIndex índice do objecto a ser removido
     * do array(filtro).
    */
    this.deleteFilterObject = function(filter, objectIndex){
        if(objectIndex === 0)
            filter.shift();    
        else 
            if(objectIndex === filter.length - 1)
                filter.pop();
            else 
                filter = filter.slice(0,objectIndex).concat(filter.slice(objectIndex + 1));    

        return filter;
    };
    /**
     * Mostra mensagem de erro, sendo ela uma string ou um objeto de erro de promise.
     */
    this.showErrorMessage = function(error) {
        console.error(error);
        var message = error;
        if (typeof error === 'object') {
            if (error.data && error.data.error) {
                message = error.data.error;
            }
            else if (error.message) {
                message = error.message;
            }
        }

        if (!message) {
            message = 'Ocorreu um erro inesperado. Entre em contato com o administrador do sistema.';
        }
        else if (message.indexOf('Exception: Exception: ') != -1) {
            message = message.replace('Exception: Exception: ', '');
        }

        ScreenService.showMessage(message, 'error');
    };
    
    this.iniciarAutocomplete = function(field){
        field.initializeField();
    };
    
    
    /**
     * Carrega os registros da widget passada quando ela estiver em modo list.
     * 
     * @param {object} widget
     */
    this.carregarWidgetOnEnterTab = function(widget) {
        if (widget.mode == "list") widget.reload();
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register("PAR_Utilities", PAR_Utilities);
});

// FILE: ./assets/js/repositories/Operador_Repository.js
var Operador_Repository = function(localStorageEngine) {
    this.retornaOperadorLogado = function() {
        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
        ];
        return localStorageEngine.download("par#/crud/operador_geral", filter);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register("Operador_Repository", Operador_Repository);
});

// FILE: ./assets/js/repositories/ParceiroNegocio_Repository.js
var ParceiroNegocio_Repository = function (restEngine) {

    this.salvarComunicacao = function(row) {
        return restEngine.post('par#/comunicaparc_formacomu/save', {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 60000);
    };
    
    this.salvarContato = function(row) {
        return restEngine.post('par#/relacionaparc_parceiro/saveContato', {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 60000);
    };

    this.salvarEndereco = function(row) {
        return restEngine.post('par#/enderecoparc_endereco/save', {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 60000);
    };

    this.salvarParceiro = function(row) {
        return restEngine.post('par#/parcnegocio/save', {
            "requestType": "Row",
            "row": row
        }, null, null, null, null, 60000);
    };

};

Configuration(function(ContextRegister){
    ContextRegister.register('ParceiroNegocio_Repository', ParceiroNegocio_Repository);
});

// FILE: ./assets/js/services/ParceiroNegocio_Service.js
var ParceiroNegocio_Service = function (ParceiroNegocio_Repository) {

    var defaultTiposParceiro = [
        "CLIENTE",
        "PROSPECT",
        "CANDIDATO",
        "PESSOA",
        "OPERADOR",
        "FUNCIONARIO",
        "FORNECEDOR",
        "TRANSPORTADORA",
        "VENDEDOR",
        "INTERLOCUTOR",
        "REVENDA"
    ];

    var tipoInscricao = {
        "F": "CPF",
        "I": "LIVRE",
        "J": "CNPJ",
        "O": "CODIGO"
    };

    this.getTipoInscricao = function(tipoDefault) {
        return tipoInscricao[tipoDefault];
    };

    this.salvarComunicacao = function(row) {
        return ParceiroNegocio_Repository.salvarComunicacao(row);
    };
    
    this.salvarContato = function(row) {
        return ParceiroNegocio_Repository.salvarContato(row);
    };

    this.salvarEndereco = function(row) {
        return ParceiroNegocio_Repository.salvarEndereco(row);
    };

    this.salvarParceiro = function(row) {
        return ParceiroNegocio_Repository.salvarParceiro(row);
    };

    this.setTipoPrincipal = function(row) {
        var tipos = row.TIPODEPARCEIRO;
        if (Array.isArray(tipos) && tipos.length > 0) {
            var i = 0;
            var tipoPrincipal = null;
            while (tipoPrincipal === null && i < defaultTiposParceiro.length) {
                if (tipos.indexOf(defaultTiposParceiro[i]) !== -1) {
                    tipoPrincipal = defaultTiposParceiro[i];
                }
                i++;
            }
            row.CDTIPOPARCPRINCIPAL = tipoPrincipal;
        }
        else {
            row.CDTIPOPARCPRINCIPAL = null;
        }
    };
};

Configuration(function(ContextRegister){
    ContextRegister.register('ParceiroNegocio_Service', ParceiroNegocio_Service);
});

// FILE: ./assets/js/controllers/ClassificacaoParceiroController.js
function ClassificacaoParceiroController(General) {
    
    this.setClassParc = function(args) {
        var widget = args;
        if (widget.isValid()) {
            widget.currentRow.NRORG = getLocalVar("NRORG") ? getLocalVar("NRORG") : " ";
            
            return General.saveCrud(widget).then(function(){
                widget.reload();
            });
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("ClassificacaoParceiroController", ClassificacaoParceiroController);
});

// FILE: ./assets/js/controllers/ParceiroNegocio_Controller.js
var ParceiroNegocio_Controller = function(ScreenService, General, ParceiroNegocio_Service, templateManager, PAR_Utilities, Operador_Repository, Endereco, localStorageEngine) {
    var selectedCountry;
    var selectedState;
    var selectedCity;

    var self = this;

    var parceiroNegocioWidget = function() {
        return templateManager.container.getWidget('parceiroNegocio');
    };

    this.alteraEstadoReadonly = function(widget) {
        var paisField = widget.getField("NMPAIS");
        var estadoField = widget.getField("NMESTADO");
        var currentRow = widget.currentRow;
        if (!paisField.value()) {
            estadoField.readOnly = true;
            currentRow.SGESTADO = null;
            currentRow.NMESTADO = null;
        }
        else {
            estadoField.readOnly = false;
        }
        this.alteraMunicipioReadonly(widget);
    };

    this.parceiroNegocioWidgetOnEnter = function(args){
        //var loginPromise = localStorageEngine.getLocalVar('loginPromise') || ZHPromise.when();
        //ScreenService.showLoader();
        //loginPromise.then(function() {
        //    ScreenService.hideLoader();
            var widget = parceiroNegocioWidget();
            if(widget.calledByBackAction) {
                widget.calledByBackAction = false;
            }
            widget.currentRow = { IDOPERATIVO: 'A' };
            PAR_Utilities.applyDefaultFilterOnEnter(widget);
        //}).catch(ScreenService.hideLoader);
        this.setResaleRules(args);
    };

    this.alteraMunicipioReadonly = function(widget) {
        var estadoField = widget.getField("NMESTADO");
        var municipioField = widget.name == "parceiroNegocio" ? widget.getField("NMMUNICIPIO") : widget.getField("NMMUNICIPIO");
        var currentRow = widget.currentRow;
        if (!estadoField.value()) {
            municipioField.readOnly = true;
            currentRow.NMMUNICIPIO = null;
            currentRow.CDMUNICIPIO = null;
            currentRow.NMMUNICIPIO = null;
        }
        else {
            municipioField.readOnly = false;
        }
        this.alteraBairroReadonly(widget);
        this.checkTransportadoraIsSelected(widget);
    };

    this.alteraBairroReadonly = function(widget) {
        var municipioField = widget.name == "parceiroNegocio" ? widget.getField("NMMUNICIPIO") : widget.getField("NMMUNICIPIO");
        var bairroField = widget.name == "parceiroNegocio" ? widget.getField("NMBAIRROTRAN") : widget.getField("NMBAIRRO");
        var currentRow = widget.currentRow;
        if (!municipioField.value()) {
            bairroField.readOnly = true;
            currentRow.NMBAIRROTRAN = null;
            currentRow.CDBAIRRO = null;
            currentRow.NMBAIRRO = null;
        }
        else {
            bairroField.readOnly = false;
        }
    };


    this.cancelarNovoParceiro = function() {
        var enderecoPrincipalWidget = templateManager.container.getWidget("enderecoPrincipal");
        enderecoPrincipalWidget.isVisible = false;
        enderecoPrincipalWidget.showAsTab = false;

        var enderecoWidget = templateManager.container.getWidget("parceiroNegocioEndereco");
        enderecoWidget.isVisible = true;
        enderecoWidget.showAsTab = true;

        var parentWidget = templateManager.container.getWidget("parceiroNegocio");
        parentWidget.list();
        parentWidget.dataSource.data.pop();
        parentWidget.currentRow = {};
        toggleWidgetsEnabled(parentWidget.widgets, true);
        ScreenService.goBack();
    };

    this.checkDataExists = function(field, sourceName, sourceParent) {
        field.reload().then(function() {
            if (!(field.dataSource.data && field.dataSource.data.length > 0)) {
                ScreenService.showMessage(ScreenService.i18n("Não há " + sourceName + " cadastrado para o " + sourceParent + " informado."));
            }
        });
    };

    this.cancelEditParceiro = function(args) {
        var parentWidget = args.owner.widget;
        if (parentWidget.isNewRow()) {
            this.cancelarNovoParceiro();
        }
        else {
            General.backOrCancel(args);
            toggleWidgetsEnabled(parentWidget.widgets, true);
        }
    };

    this.checkTransportadoraIsSelected = function(widget, editingWidget) {
        var tiposParceiro = widget.currentRow.TIPODEPARCEIRO;
        if (Array.isArray(tiposParceiro) && tiposParceiro.indexOf("TRANSPORTADORA") !== -1) {
            if (tiposParceiro.indexOf("FORNECEDOR") === -1 && editingWidget) {
                widget.currentRow.TIPODEPARCEIRO.push("FORNECEDOR");
            }
            changeTransportFieldsVisibility(widget, true);
        }
        else {
            changeTransportFieldsVisibility(widget, false);
        }
    };

    var changeTransportFieldsVisibility = function(widget, isVisible) {
        var fieldGroups = widget.fieldGroups.filter(function(fieldGroup) {
            return fieldGroup.name == "transportadora";
        });
        if (fieldGroups[0]) {
            fieldGroups[0].isVisible = isVisible;
        }
        if (!isVisible) {
            var currentRow = widget.currentRow;
            widget.getField("NRINSESTTRANFORMAT").validations = {};
            widget.getField("NRCEPTRAN").validations = {};
            widget.getField("NMPAIS").validations = {};
            widget.getField("NMESTADO").validations = {};
            widget.getField("NMMUNICIPIO").validations = {};
            widget.getField("NMBAIRROTRAN").validations = {};
            widget.getField("ENDERECOTRAN").validations = {};
            currentRow.NRINSESTTRAN = null;
            currentRow.NRCEPTRAN = null;
            currentRow.NMPAIS = null;
            currentRow.NMESTADO = null;
            currentRow.NMMUNICIPIO = null;
            currentRow.NMBAIRROTRAN = null;
            currentRow.ENDERECOTRAN = null;
        }
        else {
            widget.getField("NRINSESTTRANFORMAT").validations = {required: true};
            widget.getField("NRCEPTRAN").validations = {required: true};
            widget.getField("NMPAIS").validations = {required: true};
            widget.getField("NMESTADO").validations = {required: true};
            widget.getField("NMMUNICIPIO").validations = {required: true};
            widget.getField("NMBAIRROTRAN").validations = {required: true};
            widget.getField("ENDERECOTRAN").validations = {required: true};
        }
    };

    this.editEnderecoParceiro = function(widget) {
        General.edit(widget);
        this.alteraEstadoReadonly(widget);
        widget.getField("CDTIPOENDERECO").readOnly = (widget.currentRow.CDTIPOENDERECO == "PRINCIPAL");
    };

    this.editParceiro = function(parentWidget) {
        General.edit(parentWidget);
        toggleWidgetsEnabled(parentWidget.widgets, false);
        
        var tipoParceiroIsArray = Array.isArray(parentWidget.currentRow.TIPODEPARCEIRO);
        parentWidget.currentRow.TIPODEPARCEIRO   = tipoParceiroIsArray ? parentWidget.currentRow.TIPODEPARCEIRO   : parentWidget.currentRow.TIPODEPARCEIRO.split(", ");
        parentWidget.currentRow.NMTIPODEPARCEIRO = tipoParceiroIsArray ? parentWidget.currentRow.NMTIPODEPARCEIRO : [parentWidget.currentRow.NMTIPODEPARCEIRO];
    };

    this.enderecoAfterMoveRow = function(widget) {
        widget.actions.filter(function(action) {
            return action.name == "delete";
        })[0].isVisible = (widget.currentRow.CDTIPOENDERECO != "PRINCIPAL");
    };

    this.novoComunicacao = function(widget) {
        var parentRow = widget.parent.currentRow;
        widget.edit();
        widget.newRow();
        widget.currentRow.NRPARCNEGOCIO = parentRow.NRPARCNEGOCIO;
        //widget.currentRow.NRENDERECOPARC = '5';
        widget.currentRow.CDCOMPLECOMUPARC  = '5';
    };

    this.novoEndeParc = function(widget) {
        var parentRow = widget.parent.currentRow;
        widget.edit();
        widget.newRow();
        widget.currentRow.NRPARCNEGOCIO = parentRow.NRPARCNEGOCIO;
        widget.getField("CDTIPOENDERECO").readOnly = false;
        this.alteraEstadoReadonly(widget);
    };

    this.novoParceiro = function(widget) {
        widget.newRow();
        toggleWidgetsEnabled(widget.widgets, false);

        var enderecoWidget = templateManager.container.getWidget("parceiroNegocioEndereco");
        enderecoWidget.isVisible = false;
        enderecoWidget.showAsTab = false;

        var enderecoPrincipalWidget = templateManager.container.getWidget("enderecoPrincipal");
        enderecoPrincipalWidget.newRow();
        enderecoPrincipalWidget.enabled = true;
        enderecoPrincipalWidget.isVisible = true;
        enderecoPrincipalWidget.showAsTab = true;
        enderecoPrincipalWidget.currentRow.NMTIPOENDERECO = "Principal";
        enderecoPrincipalWidget.currentRow.CDTIPOENDERECO = "PRINCIPAL";

        this.alteraEstadoReadonly(enderecoWidget);
        this.alteraEstadoReadonly(enderecoPrincipalWidget);

        widget.currentRow.IDPESSOAFISICA = 'N';
        widget.currentRow.IDPARCFUNDIDO = 'N';
        widget.currentRow.IDINSTITUICAO = 'N';
    };

    this.nrinscricaoOnChange = function(field) {
        var currentRow = field.widget.currentRow;
        if (field.value()) {
            var validations = General.inscricaoFormat(field.value());
            var tipoInscricaoParceiro = ParceiroNegocio_Service.getTipoInscricao(validations.tipo.cod);
            if (tipoInscricaoParceiro === undefined) {
                ScreenService.showMessage(ScreenService.i18n("Ocorreu um erro ao validar a inscrição."));
            }
            else {
                currentRow.NRINSCRICAOPARCFORMAT = validations.value;
                currentRow.CDTIPOINSCRICAO = tipoInscricaoParceiro;
                currentRow.NRINSCRICAOPARC = validations.parameter;
                if (tipoInscricaoParceiro == "CPF") {
                    currentRow.IDPESSOAFISICA = 'S';
                    currentRow.IDINSTITUICAO = 'N';
                }
                else {
                    currentRow.IDPESSOAFISICA = 'N';
                    currentRow.IDINSTITUICAO = 'S';
                    if (tipoInscricaoParceiro != "CNPJ") {
                        currentRow.NRINSCRICAOPARC = validations.value;
                    }
                }
            }
        }
        else {
            currentRow.CDTIPOINSCRICAO = null;
            currentRow.NRINSCRICAOPARC = null;
        }
    };

    this.nrinscricaoEstadualOnChange = function(field) {
        var currentRow = field.widget.currentRow;
        if (field.value()) {
            var validations = General.inscricaoFormat(field.value(), true);
            var tipoInscricaoParceiro = ParceiroNegocio_Service.getTipoInscricao(validations.tipo.cod);
            if (tipoInscricaoParceiro === undefined) {
                ScreenService.showMessage(ScreenService.i18n("Ocorreu um erro ao validar a inscrição estadual."));
            }
            else {
                currentRow.NRINSESTTRANFORMAT = validations.value;
                currentRow.NRINSESTTRAN = tipoInscricaoParceiro == "LIVRE" ? validations.value : validations.parameter;
            }
        }
        else {
            currentRow.NRINSESTTRAN = null;
        }
    };

    this.parceiroAfterMoveRow = function(widget) {
        var tiposDeParceiro = widget.currentRow.TIPODEPARCEIRO;
        if (tiposDeParceiro && !Array.isArray(tiposDeParceiro)) {
            widget.currentRow.TIPODEPARCEIRO = tiposDeParceiro.split(", ");
        }
        if (!widget.isNewRow()) {
            var nrInscricao = widget.currentRow.NRINSCRICAOPARC;
            var nrInscricaoTrans = widget.currentRow.NRINSESTTRAN;
            var validations = General.inscricaoFormat(nrInscricao, true);
            widget.currentRow.NRINSCRICAOPARCFORMAT = validations.value;
            if (nrInscricaoTrans) {
                validations = General.inscricaoFormat(nrInscricaoTrans, true);
                widget.currentRow.NRINSESTTRANFORMAT = validations.value;
            }
        }
    };

    this.salvarComunicacao = function(args) {
        var currentRow = args.row;
        var widget = args.owner.widget;
        var paramsNewCode = widget.paramsNewCode;

        var keysParamsNewCode = Object.keys(paramsNewCode);
        keysParamsNewCode.forEach(function(value) {
            currentRow[value] = paramsNewCode[value];
        });

        if (widget.isValid()) {
            widget.currentRow.NRPARCNEGOCIO = widget.parent.currentRow.NRPARCNEGOCIO;
            widget.currentRow.__columns_to_return = ["NRCOMUNICAPARC", "NRORG"];
            ParceiroNegocio_Service.salvarComunicacao(currentRow).then(function(successData) {
                if (widget.isNewRow()) {
                    if (successData.dataset && successData.dataset.columnsToReturn[0]) {
                        widget.currentRow.NRCOMUNICAPARC = successData.dataset.columnsToReturn[0].NRCOMUNICAPARC;
                        widget.currentRow.NRORG = successData.dataset.columnsToReturn[0].NRORG;
                    }
                    widget.currentRow.__is_new = false;
                }
                widget.view();
            }, function(error) {
                var errorMessage = 'Timeout';
                if (error.data) {
                    errorMessage = error.data.error;
                }
                ScreenService.showMessage(errorMessage);
            });
        }
    };

    this.salvarEndereco = function(widget) {
        var currentRow = widget.currentRow;
        var paramsNewCode = widget.paramsNewCode;

        var keysParamsNewCode = Object.keys(paramsNewCode);
        keysParamsNewCode.forEach(function(value) {
            currentRow[value] = paramsNewCode[value];
        });

        if (!widget.isNewRow()) {
            currentRow.tipoEnderecoWasEdited = (widget.backupRow.CDTIPOENDERECO != currentRow.CDTIPOENDERECO);
            currentRow.CDTIPOENDERECOANTIGO = widget.backupRow.CDTIPOENDERECO;
        }

        if (widget.isValid()) {
            widget.currentRow.NRPARCNEGOCIO = widget.parent.parent.currentRow.NRPARCNEGOCIO;
            widget.currentRow.TIPODEPARCEIRO = widget.parent.parent.currentRow.TIPODEPARCEIRO;
            
            widget.currentRow.NMBAIRROENDERECO = widget.currentRow.NMBAIRRO;
            
            ParceiroNegocio_Service.salvarEndereco(currentRow).then(function(successData) {
                if (widget.isNewRow()) {
                    if(successData.dataset && successData.dataset.columnsToReturn)
                        widget.currentRow.NRENDERECOPARC = successData.dataset.columnsToReturn.NRENDERECOPARC;
                    widget.currentRow.__is_new = false;
                }
                widget.view();
            }, function(error) {
                var errorMessage = 'Timeout';
                if (error.data) {
                    errorMessage = error.data.error;
                }
                ScreenService.showMessage(errorMessage);
            });
        }
    };

    this.salvarNovoParceiro = function() {
        var parceiroWidget = templateManager.container.getWidget("parceiroNegocio");
        if (validateNewParceiro(parceiroWidget)) {
            this.salvarParceiro(parceiroWidget, true);
        }
    };

    this.salvarParceiro = function(widget, salvarEndereco) {
        var currentRow = widget.currentRow;
        var paramsNewCode = widget.paramsNewCode;

        var keysParamsNewCode = Object.keys(paramsNewCode);
        keysParamsNewCode.forEach(function(value) {
            currentRow[value] = paramsNewCode[value];
        });

        if (!widget.isNewRow()) {
            currentRow.inscricaoWasEdited = (widget.backupRow.NRINSCRICAOPARC != currentRow.NRINSCRICAOPARC);
        }

        if ((validateNewParceiro(widget)) && widget.isValid()) {
            this.nrinscricaoOnChange(widget.getField('NRINSCRICAOPARC'));
            ParceiroNegocio_Service.setTipoPrincipal(currentRow);
            ParceiroNegocio_Service.salvarParceiro(currentRow).then(function(successData) {
                toggleWidgetsEnabled(widget.widgets, true);
                if (widget.isNewRow()) {
                    widget.currentWidget = widget;

                    var enderecoPrincipalWidget = templateManager.container.getWidget("enderecoPrincipal");
                    enderecoPrincipalWidget.isVisible = false;
                    enderecoPrincipalWidget.showAsTab = false;

                    var enderecoWidget = templateManager.container.getWidget("parceiroNegocioEndereco");
                    enderecoWidget.isVisible = true;
                    enderecoWidget.showAsTab = true;
                    
                    if(successData.dataset && successData.dataset.columnsToReturn){
                        widget.currentRow.NRPARCNEGOCIO                  = successData.dataset.columnsToReturn.NRPARCNEGOCIO;
                        widget.currentRow.NRORG                          = successData.dataset.columnsToReturn.NRORG;
                        enderecoPrincipalWidget.currentRow.NRPARCNEGOCIO = successData.dataset.columnsToReturn.NRPARCNEGOCIO;
                        enderecoPrincipalWidget.currentRow.NRORG         = successData.dataset.columnsToReturn.NRORG;

                    }
                    
                    widget.currentRow.__is_new = false;
                    widget.activate();
                    
                    if (salvarEndereco) {
                        this.salvarEndereco(enderecoPrincipalWidget);
                    }
                }
                widget.view();
            }.bind(this),
            function(error) {
                var errorMessage = 'Timeout';
                if (error.data) {
                    errorMessage = error.data.error;
                }
                ScreenService.showMessage(errorMessage);
            });
        }
    };

    this.setTipoPrincipal = function(field) {
        var currentRow = field.widget.currentRow;
        ParceiroNegocio_Service.setTipoPrincipal(currentRow);
    };

    var toggleWidgetsEnabled = function(widgets, enabled) {
        widgets.forEach(function(widget) {
            widget.enabled = enabled;
        });
    };

    this.validarCepEnd = function(widget) {
        var currentRow = widget.currentRow;
        var CEP = currentRow.NRCEPENDERECO || currentRow.NRCEPTRAN;
        
        if (CEP) {
            Endereco.buscarCep(CEP).then(function(response) {
                    
                var addressFound = response.dataset.ENDERECO;
                if(addressFound) {
                    currentRow.NMPAIS           = addressFound.NMPAIS ? addressFound.NMPAIS : '';
                    currentRow.CDPAIS           = addressFound.CDPAIS ? addressFound.CDPAIS : '';
                    currentRow.NMESTADO         = addressFound.NMESTADO ? addressFound.NMESTADO : '';
                    currentRow.SGESTADO         = addressFound.SGESTADO ? addressFound.SGESTADO : '';
                    currentRow.NMMUNICIPIO      = addressFound.NMMUNICIPIO ? addressFound.NMMUNICIPIO : '';
                    currentRow.CDMUNICIPIO      = addressFound.CDMUNICIPIO ? addressFound.CDMUNICIPIO : '';
                    currentRow.NMBAIRRO         = addressFound.NMBAIRRO ? addressFound.NMBAIRRO : '';
                    currentRow.CDBAIRRO         = addressFound.CDBAIRRO ? addressFound.CDBAIRRO : '';
                    currentRow.NMBAIRROENDERECO = addressFound.NMBAIRRO ? addressFound.NMBAIRRO : '';
                    currentRow.CDLOGRADOURO     = addressFound.CDLOGRADOURO ? addressFound.CDLOGRADOURO : '';
                    currentRow.DSLOGRADOURO     = addressFound.DSLOGRADOURO ? addressFound.DSLOGRADOURO : '';
                    currentRow.DSENDERECO       = addressFound.DSENDERECO ? addressFound.DSENDERECO : '';
                    
                    this.alteraEstadoReadonly(widget);
                }
                else {
                    currentRow.CEP = '';
                }
                
                if(response.dataset.ERRO) {
                    ScreenService.showMessage(response.dataset.ERRO.MESSAGE, "warning");
                }
                
                if(response.messages) {
                    ScreenService.showMessage(response.messages[0].message);    
                }
                
            }.bind(this))
            .catch(function(error) {
                console.error(error);
                if (error.data)
                    ScreenService.showMessage(error.data.error);
                else
                    ScreenService.showMessage('Erro ao realizar a busca.');
            });
        }
    };

    var validateNewParceiro = function(parceiroWidget) {
        if (parceiroWidget.isNewRow()) {
            var enderecoWidget = templateManager.container.getWidget("enderecoPrincipal");
            if (parceiroWidget.isValid() && enderecoWidget.isValid() && enderecoWidget.currentRow.CDPAIS) {
                parceiroWidget.currentRow.ENDERECO = Util.clone(enderecoWidget.currentRow);
                parceiroWidget.currentRow.ENDERECO.TIPODEPARCEIRO = parceiroWidget.currentRow.TIPODEPARCEIRO;
                return true;
            }
            else {
                if (!parceiroWidget.isValid() && (!enderecoWidget.isValid() || !enderecoWidget.currentRow.CDPAIS)) {
                    if(enderecoWidget.isValid() && !enderecoWidget.currentRow.CDPAIS) {
                        enderecoWidget.activate();

                        enderecoWidget.isValid();

                        parceiroWidget.currentWidget = parceiroWidget;
                        parceiroWidget.activate();
                    } else {
                        enderecoWidget.isValid();
                    }
                    ScreenService.showMessage("Preencha todos os campos obrigatórios das duas telas!");
                } else {
                    if((!enderecoWidget.isValid() || !enderecoWidget.currentRow.CDPAIS) && !enderecoWidget.isActive()) {
                        enderecoWidget.activate();
                        enderecoWidget.isValid();
                    } else if(!parceiroWidget.isValid() && !parceiroWidget.isActive()) {
                        parceiroWidget.currentWidget = parceiroWidget;
                        parceiroWidget.activate();
                    }
                    ScreenService.showMessage("Preencha os campos obrigatórios!");
                }
                return false;
            }
        }
        else {
            return true;
        }
    };
    
    this.novoContato = function(widget){
        var parentRow = widget.parent.currentRow;
        
        widget.edit();
        widget.newRow();
        widget.currentRow.NRPARCNEGOCIO   = parentRow.NRPARCNEGOCIO;
        widget.currentRow.NOMEPARC        = parentRow.NMPRINCIPALPARC;
        widget.currentRow.EMAIL           = "";
        widget.currentRow.TELEFONE        = null;
        widget.currentRow.NRCOMUNICAPARC  = "";
        widget.currentRow.NRRELACIONAPARC = "";
        widget.currentRow.NRPARCNEGRELAC  = "";
        widget.currentRow.NREMAIL         = "";
        widget.currentRow.NRTELEFONE      = "";
    };
    
    this.salvarContato = function(args) {
        var currentRow = args.row;
        var widget = args.owner.widget;
        var paramsNewCode = widget.paramsNewCode;
        if(!Util.isEmptyOrBlank(widget.currentRow['TELEFONE']))
            widget.currentRow['TELEFONE'] = widget.currentRow['TELEFONE'].replace(/[^0-9]/g, '');
        var email = currentRow.EMAIL;
        var telefone = currentRow.TELEFONE;
        widget.currentRow.PARENTTYPE    = widget.parent.currentRow.CDTIPOPARCPRINCIPAL;
        if(Util.isEmptyOrBlank(widget.currentRow.NOMEPARC)){
            widget.currentRow.NOMEPARC    = widget.parent.currentRow.NMCANDIDATO;
        }
        
        var keysParamsNewCode = Object.keys(paramsNewCode);
        keysParamsNewCode.forEach(function(value) {
            currentRow[value] = paramsNewCode[value];
        });

        if (widget.isValid()) {
            if(Util.isEmptyOrBlank(email) && Util.isEmptyOrBlank(telefone)){
                ScreenService.showMessage('Preencha pelo menos uma forma de comunicação (Email ou Telefone)');    
            }else{
                ParceiroNegocio_Service.salvarContato(currentRow).then(function(successData) {
                    widget.reload().then(function(){
                        widget.view();
                        var dataset = widget.dataSource.data;
                        widget.currentRow.EMAIL = email;
                        widget.currentRow.TELEFONE = telefone;
                        widget.getField("TIPODEPARCEIRO").readOnly = false;
                        widget.getField("NMTIPORELACIONA").readOnly = false;
                        if(widget.currentRow.__is_new) ScreenService.closeSwipe();
                        if(successData.dataset.columnsToReturn){
                            widget.currentRow.NRPARCNEGRELAC = parseInt(successData.dataset.columnsToReturn.NRPARCNEGOCIO);
                            widget.currentRow.NRRELACIONAPARC = parseInt(successData.dataset.columnsToReturn.NRRELACIONAPARC);
                            this.setCurrentRow(dataset, currentRow);
                            //widget.currentRow.NREMAIL = widget.dataSource.data[length-1].NREMAIL;
                            //widget.currentRow.NRTELEFONE = widget.dataSource.data[length-1].NRTELEFONE;
                        }
                    }.bind(this));
                }.bind(this));
            }
        }
    };
    
    this.edit = function(widget){
        widget.getField("TIPODEPARCEIRO").readOnly = true;
        widget.getField("NMTIPORELACIONA").readOnly = true;
        General.edit(widget);
    };
    
    this.deleteCrud = function(widget){
        widget.currentRow.PARENTTYPE    = widget.parent.currentRow.CDTIPOPARCPRINCIPAL;
        General.deleteCrud(widget);
    };
    
    this.backOrCancel = function(args){
        args.owner.widget.getField("NMTIPORELACIONA").readOnly = false;
        General.backOrCancel(args);
    };
    
    this.setCurrentRow = function(dataset, currentRow){
        dataset.forEach(function(row){
           var keys = Object.keys(row);
           keys.forEach(function(value){
              if(value === 'NRPARCNEGRELAC'){
                  if(row[value] === currentRow.NRPARCNEGRELAC){
                      currentRow.NREMAIL = row.NREMAIL;
                      currentRow.NRTELEFONE  = row.NRTELEFONE;
                  }
              } 
           });
        });
    };
    
    /**
     * Defini a visibilidade do campo de descrição 
     * na widget de contatos pelo tipo de parceiro
     * selecionado.
     * 
     * @param {object} field field de descrição.
    */
    this.setDescriptionFieldVisibilityByPartnerType = function(field){
        
        var widget = field.widget;
        var currentRow = widget.currentRow;
        var fieldValue = currentRow[field.name];
        
        var descriptionField = widget.getField('DSCARACINPROS');    
        
        if(fieldValue === "INTERLOCUTOR" && widget.parent.currentRow.CDTIPOPARCPRINCIPAL === "PROSPECT"){
            descriptionField.isVisible = true;
        } else {
            descriptionField.isVisible = false;
        }
        
    };
    
    /**
     * Esconde o campo de descrição da widget de contatos
     * ('parceiroNegocioComunicacao').
     * 
     * @param {object} widget widget de contatos do parceiro
     * em caso.
     */
    this.hideDescriptionField = function(widget) {
        
        var descriptionField = widget.getField('DSCARACINPROS'); 
        descriptionField.isVisible = false;
        
    };
    
    /**
     * Define as regras da tela de parceiro de negócio 
     * de acordo com a revenda do operador logado.
     * 
     * @param {object} widget widget principal da tela de parceiro
     * de negócio ('parceiroNegocio').
     */
    this.setResaleRules = function(widget) {
        Operador_Repository.retornaOperadorLogado().then(function(loggedOperatorInfos) {
            loggedOperatorInfos = loggedOperatorInfos[0];
            
            this.setFiltroCampoTipoParceiro(widget, loggedOperatorInfos);
            this.setFiltroCamposBairro(loggedOperatorInfos.NRORG);
            this.setFiltroCampoFormaComunicacao(loggedOperatorInfos.NRORG);
            this.setFiltroCampoTipoRelacionamento(loggedOperatorInfos.NRORG);
            this.setAddressFieldsFilterByDefaultOrgAndOperatorOrg(templateManager.container.getWidget("WDGT_INTERLOCUTOR"), loggedOperatorInfos.NRORG);
            
            if (loggedOperatorInfos.IDTPOPERADOR === "R" && Boolean(loggedOperatorInfos.NRREVENDA)) {
                var restrictObjectsByResale = [];
                
                restrictObjectsByResale.push(widget);
                restrictObjectsByResale.push(widget.getField("NMOPERRESPONSAVEL"));
                
                PAR_Utilities.setResaleRulesInRestrictObjectsByLoggedOperatorResale(restrictObjectsByResale, loggedOperatorInfos);
            }
        }.bind(this));
        this.setInterlocutorWidgetOnParcNegocioWidget(widget);
    };
    
    this.setFiltroCampoTipoParceiro = function(widget, loggedOperatorInfos) {
        widget.getField("TIPODEPARCEIRO").dataSourceFilter[0].value = [0, loggedOperatorInfos.NRORG];
        widget.getField("TIPODEPARCEIRO").reload();
    };
    
    this.setFiltroCamposBairro = function(nrorg) {
        var wdgParceiroNegocioEndereco = templateManager.container.getWidget("parceiroNegocioEndereco");
        var wdgEnderecoPrincipal  = templateManager.container.getWidget("enderecoPrincipal");
        
        var dsfCampoBairro = wdgParceiroNegocioEndereco.getField("NMBAIRRO").dataSourceFilter;
        var indFiltroNrorg = PAR_Utilities.getObjectPositionInArrayByName(dsfCampoBairro, "NRORG");
        dsfCampoBairro[indFiltroNrorg].value = [nrorg, 0];
        
        dsfCampoBairro = wdgEnderecoPrincipal.getField("NMBAIRRO").dataSourceFilter;
        indFiltroNrorg = PAR_Utilities.getObjectPositionInArrayByName(dsfCampoBairro, "NRORG");
        dsfCampoBairro[indFiltroNrorg].value = [nrorg, 0];
    };
    
    this.setFiltroCampoFormaComunicacao = function(nrorg) {
        var wdgComunicacao = templateManager.container.getWidget("parceiroNegocioComunicacao");
        
        var dsfCampoFormaComu = wdgComunicacao.getField("NMFORMACOMU").dataSourceFilter;
        var indFiltroNrorg = PAR_Utilities.getObjectPositionInArrayByName(dsfCampoFormaComu, "NRORG");
        dsfCampoFormaComu[indFiltroNrorg].value = [nrorg, 0];
    };
    
    this.setFiltroCampoTipoRelacionamento = function(nrorg) {
        var wdgContatos = templateManager.container.getWidget("parceiroNegocioContatos");
        
        var dsfCampoTipoRelaciona = wdgContatos.getField("NMTIPORELACIONA").dataSourceFilter;
        var indFiltroNrorg = PAR_Utilities.getObjectPositionInArrayByName(dsfCampoTipoRelaciona, "NRORG");
        dsfCampoTipoRelaciona[indFiltroNrorg].value = [nrorg, 0];
    };
    
    this.changeAgenciaReadOnly = function(widget){
        var fieldAgencia = widget.getField('NMAGENCIA');
        var fieldBanco   = widget.getField('NMBANCO');
        
        fieldAgencia.clearValue();
        if (Util.isEmptyOrBlank(fieldBanco.value())) {
            fieldAgencia.readOnly = true; 
        }
        else {
            fieldAgencia.readOnly = false;
        }
        this.changeContaReadOnly(widget);
    };
    
    this.changeContaReadOnly = function(widget){
        var fieldAgencia = widget.getField('NMAGENCIA');
        var fieldConta   = widget.getField('DSCONTCORR');
        
        fieldConta.clearValue();
        if(Util.isEmptyOrBlank(fieldAgencia.value())){
           fieldConta.readOnly = true; 
        }else{
            fieldConta.readOnly = false;
        }
    };
        
    /**
     * Cria a widget de interlocutor na tela de parceiro
     * de negócio se o produto em execução for o CRM.
     * 
     * @param {object} widget widget de parceiro de negócio.
    */
    this.setInterlocutorWidgetOnParcNegocioWidget = function(widget){
        if(Boolean(templateManager.CRM_PARAMETRIZACAO)){
            //AbaInterlocutorController.setInterlocutorWidgetOnActiveWidget(widget,5);
            widget.widgets = PAR_Utilities.setElementOnArrayDefinedPosition(this.factoryInterlocutorWidget(widget), widget.widgets, 5);
            widget.activate();
        }    
    };
    
    this.wdgLogotipoAfterLoadDataSource = function(wdgLogotipo) {
        wdgLogotipo.currentRow = wdgLogotipo.dataSource.data[0];
        this.controlarAcaoExcluirLogo(wdgLogotipo);
    };
    
    this.controlarAcaoExcluirLogo = function(wdgLogotipo) {
        var actExcluir = wdgLogotipo.getAction("delete");
        actExcluir.isVisible = (!wdgLogotipo.currentRow.DSDIRLOGOMARCA) ? false : true;
    };
    
    this.wdgLogotipoOnEnter = function(wdgLogotipo) {
        wdgLogotipo.list();
        wdgLogotipo.reload();
        wdgLogotipo.view();
    };
    
    this.editarLogo = function(wdgLogotipo) {
        wdgLogotipo.backupRow = Util.clone(wdgLogotipo.currentRow);
		wdgLogotipo.edit();
		wdgLogotipo.template = "widget/form.html";
		
		var widgets = this.getWidgetsMenosLogo();
		General.changeWidgetAcessibility(widgets, false);
    };
    
    this.cancelarLogo = function(wdgLogotipo) {
        Object.keys(wdgLogotipo.backupRow).forEach(function (key) {
			wdgLogotipo.currentRow[key] = wdgLogotipo.backupRow[key];
		}); 
		wdgLogotipo.view();
		wdgLogotipo.template = "widget/form-view.html";
		
		var widgets = this.getWidgetsMenosLogo();
		General.changeWidgetAcessibility(widgets, true);
    };
    
    this.salvarLogo = function(wdgLogotipo) {
        var logo = wdgLogotipo.currentRow.DSDIRLOGOMARCA;
        if (logo instanceof Array && logo.length > 0) {
            General.widgetSave(wdgLogotipo).then(function() {
                this.controlarAcaoExcluirLogo(wdgLogotipo);
                
                var widgets = this.getWidgetsMenosLogo();
                General.changeWidgetAcessibility(widgets, true);
            }.bind(this),
            function(error) {
                console.error(error);
                if (error.data)
                    ScreenService.showMessage(error.data.error);
                else
                    ScreenService.showMessage(error);
            });
        }
        else {
            ScreenService.showMessage("Escolha um arquivo para salvar.");
        }
    };
    
    this.excluirLogo = function(wdgLogotipo) {
        General.callRestEngine("par#/parcnegocio_logotipo/excluirLogo", "Row", wdgLogotipo.currentRow).then(function() {
            wdgLogotipo.currentRow.DSDIRLOGOMARCA     = null;
            wdgLogotipo.currentRow.DSDIRLOGOMARCA_OLD = null;
            ScreenService.showMessage("Logotipo excluído com sucesso!");
            this.controlarAcaoExcluirLogo(wdgLogotipo);
        }.bind(this))
        .catch(function(error) {
            console.error(error);
            ScreenService.showMessage(error.data.error);
        });
    };
    
    this.getWidgetsMenosLogo = function() {
        var wdgLogotipo = templateManager.container.getWidget("wdgLogotipoParceiro");
        
        var widgets = wdgLogotipo.parent.widgets;
		widgets = widgets.filter(function(wid) {
		    return wid.name !== wdgLogotipo.name;
		});
		
		return widgets.concat(wdgLogotipo.parent);
    };
    
    // FUNÇÕES DA ABA INTERLOCUTOR
    this.openPopupNovoSetor = function(widget){
        var widSetor = templateManager.container.getWidget('popUpNovoSetor');
        ScreenService.openPopup(widSetor).then(function(){
            widSetor.edit();
            widSetor.template = "widget/form.html";
            widSetor.activate();
            widSetor.currentRow = {};  
            widSetor.actions = widSetor.actions.splice(0,2);
        });
    };
    
    this.saveSetor = function(widget){
        var widInter = templateManager.container.getWidget('WDGT_INTERLOCUTOR');
        widget.currentRow.__columns_to_return = ["CDSETOR"];
        
        widget.currentRow.__is_new = true;
        widget.currentRow.__new_code_column_size = 4;
        widget.currentRow.CDDEPTO = '';
        General.widgetSave(widget).then(function(response){
            widInter.currentRow.CDSETOR = response.dataset.columnsToReturn[0].CDSETOR;
            widInter.currentRow.NMSETOR = widget.currentRow.NMSETOR;
            ScreenService.showMessage('Setor adicionado com sucesso!').then(function(){
                ScreenService.closePopup(true);
                widInter.activate();
            });
        });
        
    };
    
    /**
     * Define o filtro de acordo com o local referente selecionado,
     * por exemplo, filtra apenas os municípios do estado selecionado.
     * 
     * @param {object} widget widget pai da widget de interlocutor
    */
    this.placeFilter = function(widget,field){
        var placeName = field.name;
        var currentRow = widget.currentRow;
        var filter = [field.dataSourceFilter[0]];
        var filterCode;
        
        switch(placeName){
            case "NMBAIRRO":
                filterCode = "CDMUNICIPIO";
                filter.push(this.createEqualFilter(filterCode,currentRow.CDMUNICIPIO));
                filter.push(this.createEqualFilter("SGESTADO", currentRow.SGESTADO));
                filter.push(this.createEqualFilter("CDPAIS", currentRow.CDPAIS));
            break;
            case "NMESTADO":
                filterCode = "CDPAIS";
                filter.push(this.createEqualFilter("CDPAIS", currentRow.CDPAIS));
            break;
            case "NMMUNICIPIO":
                filterCode = "SGESTADO";
                filter.push(this.createEqualFilter("SGESTADO", currentRow.SGESTADO));
                filter.push(this.createEqualFilter("CDPAIS", currentRow.CDPAIS));
            break;
            
        }
        
        field.dataSourceFilter = filter;
        field.reload();
    };
    
    /**
     * Retorna um objeto "filtro" com o operador
     * de igualdade.
     * 
     * @param {string} name campo referente para filtrar
     * 
     * @param {string || number} value valor a ser filtrado
     * 
     * @return {object} objeto como filtro.s
    */
    this.createEqualFilter = function(name,value){
        var filter = {
            "name"    : name,
            "operator": "=",
            "value"   : value
        };
            
        return filter;
    };
    
    /**
     * Verifica se o usuário modificou o país selecionado,
     * caso ele tenha alterado, a função limpa os campos
     * abaixo da hierarquia de localidade.
     * 
     * @param {object} currentRow row do interlocutor a ser
     * cadastrado.
    */
    this.checkCountryChange = function(currentRow){
        var currentCountrys = currentRow.NMPAIS.toString();
        if(typeof selectedCountry === "undefined"){
            selectedCountry = currentCountrys;
        } else if(selectedCountry !== currentCountrys){
            currentRow.NMESTADO    = "";
            currentRow.SGESTADO    = "";
            currentRow.NMMUNICIPIO = "";
            currentRow.CDMUNICIPIO = "";
            currentRow.NMBAIRRO    = "";
            currentRow.CDBAIRRO    = "";
            selectedCountry        = currentCountrys;
        }
    };
    
    /**
     * Verifica se o usuário modificou o estado selecionado,
     * caso ele tenha alterado, a função limpa os campos
     * abaixo da hierarquia de localidade.
     * 
     * @param {object} currentRow row do interlocutor a ser
     * cadastrado.
    */
    this.checkStateChange = function(currentRow){
        var currentState = currentRow.NMESTADO.toString();
        if(typeof selectedState === "undefined"){
            selectedState = currentState;
        }else if(selectedState !== currentState){
            currentRow.NMMUNICIPIO = "";
            currentRow.CDMUNICIPIO = "";
            currentRow.NMBAIRRO    = "";
            currentRow.CDBAIRRO    = "";
            selectedState          = currentState;
        }
    };
    
    /**
     * Verifica se o usuário modificou o cidade selecionado,
     * caso ele tenha alterado, a função limpa os campos
     * abaixo da hierarquia de localidade.
     * 
     * @param {object} currentRow row do interlocutor a ser
     * cadastrado.
    */
    this.checkCityChange = function(currentRow){
        var currentMunicipality = currentRow.NMMUNICIPIO.toString();
        
        if(typeof selectedCity === "undefined"){
            selectedCity = currentMunicipality;
        }else if(selectedCity !== currentMunicipality){
            currentRow.NMBAIRRO = "";
            currentRow.CDBAIRRO = "";
            selectedCity        = currentMunicipality;
        }
    };
    
    /**
     * Define o bloqueio do campo de estado de acordo 
     * com a hierarquia de localidade.
     * 
     * @param {object} wodget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.alteraEstadoReadonly = function(widget) {
        //Tratando se é filtro ou edit
        if(widget.mode === "edit"){
            var paisField   = widget.getField("NMPAIS");
            var estadoField = widget.getField("NMESTADO");
            var currentRow  = widget.currentRow;
            if (!paisField.value()) {
                estadoField.readOnly = true;
                currentRow.SGESTADO = "";
                currentRow.NMESTADO = "";
            } else {
                estadoField.readOnly = false;
            }
            this.alteraMunicipioReadonly(widget);
        }
    };
    
    /**
     * Define o bloqueio do campo de município de acordo 
     * com a hierarquia de localidade.
     * 
     * @param {object} wodget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.alteraMunicipioReadonly = function(widget) {
        //Tratando se é filtro ou edit
        if(widget.mode === "edit"){
            var estadoField    = widget.getField("NMESTADO");
            var municipioField = widget.getField("NMMUNICIPIO");
            var currentRow     = widget.currentRow;
            if (!estadoField.value()) {
                municipioField.readOnly = true;
                currentRow.NMMUNICIPIO = "";
                currentRow.CDMUNICIPIO = "";
                currentRow.NMMUNICIPIO = "";
            } else {
                municipioField.readOnly = false;
            }
        }
        this.alteraBairroReadonly(widget);
    };
    
    /**
     * Define o bloqueio do campo de bairro de acordo 
     * com a hierarquia de localidade.
     * 
     * @param {object} wodget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.alteraBairroReadonly = function(widget) {
        //Tratando se é filtro ou edit
        if(widget.mode === "edit"){
            var municipioField = widget.getField("NMMUNICIPIO");
            var bairroField    = widget.getField("NMBAIRRO");
            var currentRow     = widget.currentRow;
            if (!municipioField.value()) {
                bairroField.readOnly = true;
                currentRow.NMBAIRRO = "";
                currentRow.CDBAIRRO = "";
                currentRow.NMBAIRRO = "";
            } else {
                bairroField.readOnly = false;
            }
        }
    };
    
    /**
     * Define o filtro dos campos de endereço
     * pela organização padrão do sistema e
     * pela organização do operador logado.
     * 
     * @param {object} widget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.setAddressFieldsFilterByDefaultOrgAndOperatorOrg = function(widget, nrorg){
        var addressFields = [widget.getField('NMPAIS'), widget.getField('NMESTADO'), widget.getField('NMMUNICIPIO'), widget.getField('NMBAIRRO')];
        addressFields.forEach(function(field){
            this.setAddressFieldFilterValue(field, nrorg);
        }.bind(this));
    };
    
    /**
     * Define o filtro do campo recebido como parâmtro
     * para buscar registros da organização padrão e
     * da organização do operador logado.
     * 
     * @param {object} field campo de endereço a ser
     * filtrado.
    */
    this.setAddressFieldFilterValue = function(field, nrorg){
        var fieldFilter = field.dataSourceFilter;
        var nrorgFilterIndex = PAR_Utilities.getObjectPositionInArrayByName(fieldFilter, "NRORG");
        
        if(Util.isUndefined(nrorgFilterIndex))
            nrorgFilterIndex = field.dataSourceFilter.push({name: "NRORG", operator: "IN", value: ""}) - 1;
        
        fieldFilter[nrorgFilterIndex].value = [nrorg, 0];
    };
    
    /**
     * Define os valores iniciais do cadastro de interlocutor.
     * 
     * @param {object} widget widget de interlocutor ('WDGT_INTERLOCUTOR').
    */
    this.setInterlocutorWidgetDefaultValues = function(widget){
        var currentRow           = widget.currentRow;
        var parentWidget         = widget.parent;
        if (currentRow.NRPARCNEGOCIO == null){
            currentRow.NRPARCNEGOCIO = parentWidget.currentRow.NRPARCNEGOCIO;
        }
    };
    
    /**
     * Salve um registro de interlocutor.
     * 
     * @param {object} widget widget de interlocutor('WDGT_INTERLOCUTOR').
    */
    this.saveInterlocutor = function(widget){
        var currentRow = widget.currentRow;
        this.removeInterlocutorPhoneMask(widget);
        if(Util.isEmptyOrBlank(currentRow.NRCPFINTERLOC))
            currentRow.NRCPFINTERLOC = '';
        
        currentRow.NRCPFINTERLOC = currentRow.NRCPFINTERLOC.replace(/[^0-9]/g, '');
        widget.currentRow.columnsToReturn = ["NRSEQINTERLOCUTOR"];
        General.widgetSave(widget).then(function(response){
            if(response.dataset && response.dataset.columnsToReturn && response.dataset.columnsToReturn.length > 0)
                currentRow.NRSEQINTERLOCUTOR = response.dataset.columnsToReturn[0].NRSEQINTERLOCUTOR;
        }).catch(function(error){
            PAR_Utilities.showErrorMessage(error);
        });
    };
    
    /**
     * Retira a máscara dos telefones digitados
     * na currentRow.
     * 
     * @param {object} widget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.removeInterlocutorPhoneMask = function(widget){
        var currentRow  = widget.currentRow;
        var phoneFields = ["NRTELEFONE1", "NRTELEFONE2", "NRTELEFONE3", "NRTELEFONE4"];
        
        phoneFields.forEach(function(phone){
            if(currentRow[phone])
                currentRow[phone] = currentRow[phone].replace(/[^0-9]/g, '');    
        });
    };
    
    /**
     * Abre um ambiente para edição de um
     * interlocutor.
     * 
     * @param {object} widget widget de interlocutor
     * ('CRM_INTERLOCUTOR').
    */
    this.editInterlocutor = function(widget){
        General.widgetEdit(widget);
        this.checkAddressFieldsPermission(widget);
        this.mudaTipoSignatario();
        widget.getField("NMINTERLOCUTOR").readOnly = widget.currentRow.IDNOMEVINDODAAPI == "S";
        //widget.getField("NMINTERLOCUTOR").readOnly = false;
    };
    
    /**
     * Define as permissões dos campos de endereço,
     * se eles estarão bloqueados ou não.
     * 
     * @param {object} widget widget de interlocutor
     * ('CRM_INTERLOCUTOR').
    */
    this.checkAddressFieldsPermission = function(widget){
        this.alteraEstadoReadonly(widget);
        this.alteraMunicipioReadonly(widget);
        this.alteraBairroReadonly(widget);    
    };
    
    /**
     * Preenche os campos de endereço pelo CEP
     * digitado.
     * 
     * @param {object} widget widget de interlocutor
     * ('WDGT_INTERLOCUTOR').
    */
    this.fillAddressByCep = function(widget){
        var currentRow = widget.currentRow;
        var cep = currentRow.NRCEPINTERLOCUTOR;
        
        if(cep.length === 8){
            Endereco.buscarCep(cep).then(function(response){
                var foundAddress              = response.dataset.ENDERECO;
                currentRow.NMPAIS             = "Brasil";
                currentRow.CDPAIS             = "0055";
                currentRow.NMESTADO           = foundAddress.NMESTADO;
                currentRow.SGESTADO           = foundAddress.SGESTADO;
                currentRow.NMMUNICIPIO        = foundAddress.NMMUNICIPIO;
                currentRow.CDMUNICIPIO        = foundAddress.CDMUNICIPIO;
                currentRow.NMBAIRRO           = foundAddress.NMBAIRRO;
                currentRow.CDBAIRRO           = foundAddress.CDBAIRRO;
                currentRow.DSENDEINTERLOCUTOR = foundAddress.DSENDERECO;
                this.checkAddressFieldsPermission(widget);
                
                /*TODO Encontrar um jeito melhor de fazer o autocomplete atualizar os fields para os dados da currentRow*/
                angular.element('#footer').click();
            
            }.bind(this))
            .catch(function(error){
                ScreenService.showMessage(ScreenService.i18n("Endereço não encontrado."));
            });    
        } else {
            ScreenService.showMessage(ScreenService.i18n("CEP Inválido"));
        }
        
    };
    
     this.completaPercentual = function(widget){
        
        General.widgetNewRow(widget);
        
        var params = {
           "NRPARCNEGOCIO" : widget.currentRow.NRPARCNEGOCIO
        };
        
        General.callRestEngine('par#/parcnegocio/completaPercentual', "Row", params).then(function(response){  
             
            var percentual = response.dataset.completaPercentual.PERCENTUAL;
            
            widget.currentRow.NRPERCENTINDICE = percentual;
            
        });
    
    };
    
    
    this.validaSaveIndFinanceiro = function(widget){
        
        var params = {
           "NRPARCNEGOCIO" : widget.currentRow.NRPARCNEGOCIO,
           "NRPERCENTINDICE" : widget.currentRow.NRPERCENTINDICE
        };
        
        General.callRestEngine('par#/parcnegocio/validaPercentual', "Row", params).then(function(response){  
             
            var percentual = response.dataset.validaPercentual.PERCENTUAL;
            
            if (percentual < 100) {
                
                
                General.widgetSave(widget).then(function(response){
                    widget.reload();
                    
                    ScreenService.showMessage("Atenção! <br><br> A soma dos percentuais de utilização dos índices financeiros não está totalizando os 100% necessários para realização do reajuste dos contratos.<br><br> Só será possível realizar o reajuste quando a soma dos percentuais de uso dos índices financeiros estiverem somando 100%.");
                });
            }
            else if (percentual == 100) {
                
                General.widgetSave(widget).then(function(response){
                    
                    widget.reload();
                    
                });
                
            }
            else{
                
                ScreenService.showMessage("Operação Bloqueada! A soma dos percentuias de utilização dos índices financeiros ultrapassou o total de 100%.");
            }
            
        }.bind(this));
        
    };
    
    
    this.validaExibeIndFinanceiro = function(widget){
    
        var $tipoParc = widget.currentRow.CDTIPOPARCPRINCIPAL;
        var indicesFinanceirosPadroes = templateManager.container.getWidget("indicesFinanceirosPadroes");
    
        if ($tipoParc == 'CLIENTE') {
            
             indicesFinanceirosPadroes.isVisible = true;
        }
        else{
            
             indicesFinanceirosPadroes.isVisible = false;
        }
    
    };
    
    this.mudaTipoSignatario = function(){
    
        var wdgInterlocutor = templateManager.container.getWidget("WDGT_INTERLOCUTOR");
        var $tipoSignatario = wdgInterlocutor.currentRow.IDTPSIGNATARIO;
    
        if ($tipoSignatario == 'N') {
            
             wdgInterlocutor.getField("NRCPFINTERLOC").validations = {};
             wdgInterlocutor.getField("EMAILINTERLOC").validations = {};
             wdgInterlocutor.getField("DTNASCIMENTO").validations = {};
        }
        else{
            
             wdgInterlocutor.getField("NRCPFINTERLOC").validations = {"required": true};
             wdgInterlocutor.getField("EMAILINTERLOC").validations = {"required": true};
             wdgInterlocutor.getField("DTNASCIMENTO").validations = {"required": true};
        }
    
    };
    
    this.saveParamFatura = function(widget){
        
        var parceiroWidget = templateManager.container.getWidget("parceiroNegocio");
        
        widget.currentRow.NRPARCNEGOCIO = parceiroWidget.currentRow.NRPARCNEGOCIO;
        
        General.callRestEngine('par#/parcnegocio/saveParamFatura', "Row", widget.currentRow).then(function(){  
           
            widget.reload();
            widget.view();
            toggleWidgetsEnabled(parceiroWidget.widgets, true);
            
        }).catch(function(error) {
            
            PAR_Utilities.showErrorMessage(error);
        });
    };
    
    this.cancelParamFatura = function(widget){
        General.widgetCancelForm(widget);
        
        var parceiroWidget = templateManager.container.getWidget("parceiroNegocio");
        toggleWidgetsEnabled(parceiroWidget.widgets, true);
    };
    
    this.parcParamFatAfterLoadDataSorce = function(widget){
        
        var parceiroWidget = templateManager.container.getWidget("parceiroNegocio");
        
        var params = {
           "NRPARCNEGOCIO" : parceiroWidget.currentRow.NRPARCNEGOCIO
        };
        
        General.callRestEngine('par#/parcnegocio/recebeParamFatura', "Row", params).then(function(response){  
           
            if (response.dataset.recebeParamFatura.dados != null){
                var dados = response.dataset.recebeParamFatura.dados[0];
                
                widget.currentRow.NRPARAMFATURA  = dados.NRPARAMFATURA;
                widget.currentRow.NRPARCNEGOCIO  = dados.NRPARCNEGOCIO;
                widget.currentRow.CDFILIALFATURA = dados.CDFILIALFATURA;
                widget.currentRow.NMFILIALFATURA = dados.NMFILIALFATURA;
                widget.currentRow.CDSERIENF      = dados.CDSERIENF;
                widget.currentRow.DSSERIE        = dados.DSSERIE;
                widget.currentRow.CDEMPRESA      = dados.CDEMPRESA;
                widget.currentRow.CDINSCESTA     = dados.CDINSCESTA;
                widget.currentRow.CDINSCMUNI     = dados.CDINSCMUNI;
                widget.currentRow.IDATIVO        = dados.IDATIVO;
                widget.currentRow.NRORG          = dados.NRORG;
                widget.currentRow.DTINCLUSAO     = dados.DTINCLUSAO;
                widget.currentRow.CDOPERINCLUSAO = dados.CDOPERINCLUSAO;
                widget.currentRow.NRORGINCLUSAO  = dados.NRORGINCLUSAO;
                widget.currentRow.DTULTATU       = dados.DTULTATU;
                widget.currentRow.CDOPERULTATU   = dados.CDOPERULTATU;
                widget.currentRow.NRORGULTATU    = dados.NRORGULTATU;
              
            }
            else {
                
                widget.currentRow.NRPARAMFATURA  = null;
                widget.currentRow.NRPARCNEGOCIO  = null;
                widget.currentRow.CDFILIALFATURA = null;
                widget.currentRow.NMFILIALFATURA = null;
                widget.currentRow.CDSERIENF      = null;
                widget.currentRow.DSSERIE        = null;
                widget.currentRow.CDEMPRESA      = null;
                widget.currentRow.CDINSCESTA     = null;
                widget.currentRow.CDINSCMUNI     = null;
                widget.currentRow.IDATIVO        = null;
                widget.currentRow.NRORG          = null;
                widget.currentRow.DTINCLUSAO     = null;
                widget.currentRow.CDOPERINCLUSAO = null;
                widget.currentRow.NRORGINCLUSAO  = null;
                widget.currentRow.DTULTATU       = null;
                widget.currentRow.CDOPERULTATU   = null;
                widget.currentRow.NRORGULTATU    = null;
            }
            
            widget.view();
            
        }).catch(function(error) {
            
            PAR_Utilities.showErrorMessage(error);
        });
        
    };
    
    this.prepararRow = function(widget){
        try{
            widget.getField("NMINTERLOCUTOR").readOnly = false;
            General.widgetNewRow(widget);
        }catch(err){
            ScreenService.showMessage(ScreenService.i18n(`Operação bloqueada! <br> ${err}`));
        }
    };
    
    //#region VALIDAÇÂO CPF _ WEBSERVICE
    
    /**
     * Esse procedimento é responsável por fazer toda a funcionalidade de substituir o nome do interlocutor baseado no CPF
     * @params{Widget} widget da popup do interlocutor
    */
    this.fazerConsultaETrocarNome = function(widget){
        //aplicar a validaçãqo do general
        widget.getField("NMINTERLOCUTOR").readOnly = false;
        widget.currentRow.IDNOMEVINDODAAPI = "N";
        var required = (widget.getField("NRCPFINTERLOC").validations) ? widget.getField("NRCPFINTERLOC").validations.required : false;
        var cpfValido = General.validateCPF(widget.currentRow.NRCPFINTERLOC);
        var data = widget.currentRow.DTNASCIMENTO; 
        var cpf = widget.currentRow.NRCPFINTERLOC; 
        if(cpfValido || required ){
            this.pegarValidacaoCPF().then(function(response){
                if(response.dataset.resulted.IDVALIDACPFRF != 'N'){
                    this.efetuarConsulta(widget, cpfValido);
                }else{
                    if(data && cpf){
                        ScreenService.confirmMessage("Deseja buscar o nome da pessoa de acordo com o CPF e Data de Nascimento?", "question", function(){
                            this.efetuarConsulta(widget, cpfValido);
                        }.bind(this));
                    }
                }
            }.bind(this));
        }else{
           if(required ) 
                ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br>O valor preenchido no CPF não representa um valor válido."));
        }
    };
    
   /**
     * Função responsável por fazer a consulta do CPF e alterar o nome do campo da popup
    */
    this.efetuarConsulta = function(popUp, cpfValido = false){
        var data = popUp.currentRow.DTNASCIMENTO; 
        var cpf = popUp.currentRow.NRCPFINTERLOC; 
        if (data && cpf) {
            if (cpfValido) {
                this.validaNaAPIAntes(data,cpf); 
            }
            else {
                ScreenService.showMessage(ScreenService.i18n("Operação bloqueada! <br><br>O CPF informado é inválido."));
            }
        }
        
        /*if(cpfValido && data && cpf)
            this.validaNaAPIAntes(data,cpf); 
        else{
            ScreenService.showMessage(ScreenService.i18n(`Operação bloqueada! <br><br> ${!cpfValido 
            ? "O valor do campo do CPF é inválido, favor inserir um valor válido para o CPF."
            : "O campo de data deve possuir algum valor, favor inserir um valor no campo Data de Nascimento para efetuar a consulta."
            }`));
        }*/
    };
    
    
    /**
     * Essa função vê o estado da consulta automática do CPF parametrizada na tela de Parametrizações SM
     * @returns {Promise} Promise da consulta à tela de Parametrizações
    */
    this.pegarValidacaoCPF = function(){
        return General.callRestEngine("product#/fct_parametrogeral_widget/pegarValidacaoCPF", "Row", {});
    };
    
    /**
     * Esse procedimento efetua a validação e retornar a resposta
     * @params {string} data de nascimento
     * @params {string} número de cpf
    */
    this.validaNaAPIAntes = function(data, cpf){
        //split para separar a data das horas
        var params = {
            "DTNASCIMENTO" : data,
            "NRCPFINTERLOC" : cpf
        };
        General.callRestEngine("product#/consultaDadosCPF", "Row", params).then(function(response){
            var obj = response.dataset.CONSULTA;
            if(obj.Status){
               this.alterarNomeInterlocutor(obj.Nome);
            }else{
                ScreenService.showMessage(ScreenService.i18n(`Operação bloqueada! <br><br>Falha na integração com a API de validação do CPF: <br> ${obj.Mensagem}`));
            }
        }.bind(this)).catch(function(err){
            ScreenService.showMessage(ScreenService.i18n(err));
        });
    };
    
    /**
     * Procedimento responsável por substituir o campo de nome e transformar ele em readonly, chamada somente em uma requisição bem sucedida
     * @params {string} nome a ser substituido
     * @params {Widget} widget da popup do interlocutor, pode ser opcional
    */
    this.alterarNomeInterlocutor = function(nome, wdg = null){
        if(wdg == null)
            wdg  = templateManager.container.getWidget("WDGT_INTERLOCUTOR");
        wdg.currentRow.NMINTERLOCUTOR = nome;
        wdg.getField("NMINTERLOCUTOR").readOnly = true;
        wdg.currentRow.IDNOMEVINDODAAPI = "S";
    };

    //#endregion
    
    /**
     * Limpa o campo de data
     * 
     * @param {string} no do campo, mesmo nome presente no current row.
    */
    this.clearCalendarField = function(nomeDoCampo){
        setTimeout(function(){
            var campo = document.querySelector(`.zh-field-${nomeDoCampo}`);
            if(campo){
                var filhos = campo.children;
                if(filhos){
                    var fechar = filhos[1].children[0];
                    if(fechar){
                        fechar.click();
                    }
                }
            }
        }, 250);

    };
};

Configuration(function (ContextRegister) {
    ContextRegister.register('ParceiroNegocio_Controller', ParceiroNegocio_Controller);
});

